/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package advanced;

import java.io.IOException;

import jama.Matrix;
import jgromacs.analysis.Distances;
import jgromacs.analysis.Dynamics;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;
import jgromacs.io.IOMath;

/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to compare residue contacts in different conformations
 *  The code finds those residue contacts which are present both in the first 
 *  and in the last frame of the trajectory and have a contact probability 
 *  larger than 0.8 in the whole trajectory. (Contacts are calculated between
 *  the closest non-hydrogen atoms and a distance cutoff of 0.6 nm is used.)
 *  
 */
public class CompareContacts {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
		
		// Distance cutoff defining a contact:
		double cutoff = 0.6;
		
		try {
		// Reading first structure and trajectory from file		
		structure = IOData.readStructureFromGRO("dataset//example_protein.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_protein.dxtc");
		
		// Extracting first frame and last frame
		Structure firstFrame = trajectory.getFirstFrameAsStructure();
		Structure lastFrame = trajectory.getLastFrameAsStructure();
		
		// Calculating contact matrices of the first and last frames:
		Matrix C_first = Distances.getContactMatrix(firstFrame, Distances.CLOSESTHEAVY, cutoff);
		Matrix C_last = Distances.getContactMatrix(lastFrame, Distances.CLOSESTHEAVY, cutoff);
		
		// Calculating contact probability matrix
		Matrix P = Dynamics.getContactProbabilityMap(trajectory, cutoff);
		
		// Comparing contact matrices and contact probability matrix
		int dim = C_first.getRowDimension();
		Matrix R = new Matrix(dim,dim,0);
		for (int i = 0; i < dim; i++) {
			for (int j = i; j < dim; j++) {
				if ((C_first.get(i, j)==1)&(C_last.get(i, j)==1)&(P.get(i, j)>0.8)) {
					R.set(i, j, 1);
					R.set(j, i, 1);
				}
			}
		}
		
		// In the resulting matrix R the entry R_ij is 1 if the contact probability
		// is more than 0.8 and the contact is found both in the first and last frames.
		// R_ij is 0 if any of these requirements is not satisfied.
		
		IOMath.writeMatrixToFile(R, "result.txt");
		
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
