/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package advanced;

import java.io.IOException;

import jgromacs.analysis.Dynamics;
import jgromacs.analysis.Similarity;
import jgromacs.data.FrameIndexSet;
import jgromacs.data.PointList;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;

/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to select and compare conformational ensembles
 *  The code first calculates the medoid structure of a trajectory based on
 *  the RMSD similarity measure. Then it extracts those simulation frames which
 *  are similar to the medoid structure (using a RMSD similarity cutoff of 0.06).
 *  Finally it compares the resulting conformational ensemble and the total
 *  conformational ensemble of the trajectory by calculating the ensemble averaged
 *  RMSD between the two ensembles.
 *  
 */
public class CompareEnsembles {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
		
		try {
		// Reading structure and trajectory from file	
		structure = IOData.readStructureFromGRO("dataset//example_calpha.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_calpha.dxtc");
		
		// Calculating the medoid frame using the cRMSD measure
		PointList medoid = Similarity.getMedoidRMSD(trajectory);
		
		// Extracting simulation frames that are similar to the medoid frame	
		FrameIndexSet frameset = Similarity.findSimilarFramesRMSD(trajectory, medoid, 0.06);
		Trajectory trajectory2 = trajectory.getSubTrajectory(frameset);
		
		// Calculating ensemble averaged RMSD between the two ensembles
		double eRMSD = Dynamics.getEnsembleAveragedRMSD(trajectory, trajectory2);
		
		System.out.println("eRMSD: "+eRMSD);
		
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
}
