/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package advanced;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import jama.Matrix;
import jgromacs.analysis.Distances;
import jgromacs.analysis.Dynamics;
import jgromacs.data.IndexSet;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;

/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to calculate the dynamical network of a protein (Sethi et al.)
 *  The code calculates the dynamical network based on a trajectory and converts 
 *  the output into a Pajek (network visualization program) input file
 *  
 */
public class DynamicalNetwork {

	public static void main(String[] args) throws IOException {
		Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
		Trajectory sim = IOData.readDumpedTrajectory(s, "dataset//example_protein.dxtc");
		int d = sim.getNumberOfResidues();
		Matrix Contact = Distances.getFrequencyContactMatrix(sim, Distances.CLOSESTHEAVY, 0.45, 0.75);   
		  IndexSet alphaCarbons = s.getAlphaCarbonIndexSet();
		  sim = sim.getSubTrajectory(alphaCarbons);
		  Matrix Correlation = Dynamics.getAtomicCorrelationMatrix(sim);
		  Matrix W = new Matrix(d,d,0);
		  for (int i = 0; i < d; i++) {
		    for (int j = i+1; j < d; j++) {
		        if (Contact.get(i,j)==1) W.set(i, j, Math.abs(Correlation.get(i, j)));
		        else W.set(i,j,-1); 
		        W.set(j, i, W.get(i, j));  
		    }  
		 } 

	     writeToPajekFile(W, "example_pajek.net"); // Converting and writing to Pajek file
	

		 // One possible way to visualize the output in Pajek:
		 // 1. Import the example_pajek.net file using the menu File\Network\Read
	     // 2. Draw the network with the Draw\Draw menu
	     // 3. Create a different network layout with the Layout\Energy\Kamada-Kawai\Free menu
	     // 4. Configure the size, color and label of vertices/edges in the Options menu
	     // Please note that the network calculated here is not identical to the one presented in the paper 
	}
	
	// Converting to Pajek input file format
	private static void writeToPajekFile(Matrix W, String filename){
		int d = W.getRowDimension();
		String str = "*Vertices "+d+"\n";
		for (int i = 0; i < d; i++) {
	    	   str += (i+1)+" \"Res"+(i+1)+"\"\n";
	    }
	    str += "*Edges\n";
	    for (int i = 0; i < d; i++) {
		 for (int j = i+1; j < d; j++) {
			double w = W.get(i, j);
		    if (w!=(-1)) str+=(i+1)+" "+(j+1)+" "+w+"\n";
		 }
	    }
	   	writeToFile(str, filename);
	}
	
	
	 private static void writeToFile(String s, String filename){
	   	File file = new File(filename);
	   	try {
	   	   BufferedWriter output =  new BufferedWriter(new FileWriter(file));
	   	   try {     
	   			output.write(s);
	   	   }
	   	   finally {
	   		    output.close();
	       }
	   	}catch (IOException ex){
	   	   ex.printStackTrace();
	    }
	 } 
		
}
