/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package advanced;

import java.io.IOException;
import java.util.ArrayList;
import jgromacs.analysis.Distances;
import jgromacs.analysis.Dynamics;
import jgromacs.analysis.GNM;
import jgromacs.data.PointList;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;


/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to compare fluctuation profiles from MD and GNM
 *  The code calculates the RMSF profile from MD data and the
 *  MSF profile based on a GNM model and then calculates the
 *  correlation coefficient of the two profiles.
 *  
 */
public class MDvsGNM {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
			
		try {
		// Reading structure and trajectory from file	
		structure = IOData.readStructureFromGRO("dataset//example_calpha.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_calpha.dxtc");
		
		// Calculating RMSF profile from MD data
		PointList firstFrame = trajectory.getFirstFrameAsPointList();
		ArrayList<Double> RMSF_MD = Dynamics.getRMSFaroundTheMeanProfile(trajectory, firstFrame);
		
		// Calculating MSF profile from GNM data
		GNM gnm = new GNM(structure, 0.7, Distances.ALPHACARBON);
		ArrayList<Double> MSF_GNM = gnm.getMSFProfile();
		
		// Calculating correlation between the two profiles
		double sum_MD = 0;
		double sum_GNM = 0;
		for (int i = 0; i < RMSF_MD.size(); i++) {
			sum_MD+=RMSF_MD.get(i);
			sum_GNM+=MSF_GNM.get(i);
		}
		sum_MD/=RMSF_MD.size(); 
		sum_GNM/=RMSF_MD.size();
		double A = 0;
		double B = 0;
		double C = 0;
		for (int i = 0; i < RMSF_MD.size(); i++) {
			A+=(RMSF_MD.get(i)-sum_MD)*(MSF_GNM.get(i)-sum_GNM);
			B+= Math.pow(RMSF_MD.get(i)-sum_MD, 2);
			C+= Math.pow(MSF_GNM.get(i)-sum_GNM, 2);
		}
		double r = A/Math.sqrt(B*C);
		
		System.out.println("Correlation coefficient: "+r);
		
		} catch (IOException e) {
			e.printStackTrace();
		}
	
	}
	
}
