/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package analysis;

import java.io.IOException;
import java.util.ArrayList;

import jama.Matrix;
import jgromacs.analysis.Angles;
import jgromacs.data.Angle;
import jgromacs.data.Point3D;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;
/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to calculate dihedrals and other angles of interest
 * 
 */
public class Dihedrals {

	
	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
		
		try {	
		structure = IOData.readStructureFromGRO("dataset//example_protein.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_protein.dxtc");
		
		// Calculating the angle between two vectors
		Point3D v1 = new Point3D(2,2,-2);
		Point3D v2 = new Point3D(-2,2,1);
		Angle angle1 = Angles.getAngleBetweenVectors(v1, v2);
		
		// Getting and setting the angle in degrees or radians
		System.out.println(angle1.getInDegrees()); // Degrees
		System.out.println(angle1.getInRadians()); // Radians
		angle1.setInRadians(Math.PI/2);
		angle1.setInDegrees(36.8);
		
		// Calculating the angle between two plains defined by points A,B,C and points B,C,D
		Point3D A = new Point3D(2,2,-2);
		Point3D B = new Point3D(-2,2,1);
		Point3D C = new Point3D(1,1,1);
		Point3D D = new Point3D(1.5,4,1);
		Angle angle2 = Angles.getAngleBetweenPlanes(A, B, C, D);
	
		// Calculating backbone dihedral angles
		Angle phi = Angles.getDihedralPhi(structure, 13); // Phi
		Angle psi = Angles.getDihedralPsi(structure, 13); // Psi
		Angle omega = Angles.getDihedralOmega(structure, 13); // Omega
		
		// Calculating sidechain dihedral angles
		Angle chi1 = Angles.getDihedralChi1(structure, 13); // Chi1
		Angle chi2 = Angles.getDihedralChi2(structure, 13); // Chi2
		Angle chi3 = Angles.getDihedralChi3(structure, 13); // Chi3
		Angle chi4 = Angles.getDihedralChi4(structure, 13); // Chi4
		Angle chi5 = Angles.getDihedralChi5(structure, 13); // Chi5
		
		System.out.println(chi2);
		
		// Extracting time series of backbone dihedral angles 
		ArrayList<Angle> phi_timeSeries = Angles.getDihedralPhiTimeSeries(trajectory, 13);
		ArrayList<Angle> psi_timeSeries = Angles.getDihedralPsiTimeSeries(trajectory, 13);
		ArrayList<Angle> omega_timeSeries = Angles.getDihedralOmegaTimeSeries(trajectory, 13);
		
		// Extracting time series of sidechain dihedral angles 
		ArrayList<Angle> chi1_timeSeries = Angles.getDihedralChi1TimeSeries(trajectory, 13);
		ArrayList<Angle> chi2_timeSeries = Angles.getDihedralChi2TimeSeries(trajectory, 13);
		ArrayList<Angle> chi3_timeSeries = Angles.getDihedralChi3TimeSeries(trajectory, 13);
		ArrayList<Angle> chi4_timeSeries = Angles.getDihedralChi4TimeSeries(trajectory, 13);
		ArrayList<Angle> chi5_timeSeries = Angles.getDihedralChi5TimeSeries(trajectory, 13);
		
		// Extracting time series of torsion angle defined by four atoms 
		ArrayList<Angle> torsion_timeSeries = Angles.getTorsionAngleTimeSeries(trajectory, 100, 101, 102, 103);
		
		// Calculating the Ramachandran plot
		Matrix ramachandranPlot = Angles.getRamachandranPlot(structure);
		
		} catch (IOException e) {
			
			e.printStackTrace();
		}
	}
	
}
