/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package analysis;

import java.io.IOException;

import jama.Matrix;
import jgromacs.analysis.Dynamics;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;

/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to do miscellaneous things about dynamics
 * 
 */

public class DynamicsMisc {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
		
		try {
		// Reading structure and trajectory from file	
		structure = IOData.readStructureFromGRO("dataset//example_calpha.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_calpha.dxtc");
		
		// Complex ideas implemented from the literature:
		
		// Calculating the dynamical network of a protein as defined by Sethi et al.
		// (Dynamical networks in tRNA:protein complexes, 2009, PNAS)
		Matrix Adjacency = Dynamics.getDynamicalNetwork(trajectory, 0.6, 0.75); // weighted adjacency matrix of dynamical network
		
		// Calculating the structural radius of the conformational ensemble sampled in the simulation
		// as defined by Kuzmanic and Zagrovic 
		// (Determination of Ensemble-Average Pairwise Root Mean-Square Deviation from Experimental B-Factors, 2010, Biophysical Journal)
		double structuralRadius = Dynamics.getStructuralRadius(trajectory);

		// Calculating the ensemble averaged RMSD between two conformational ensembles 
		// as defined by Brschweiler 
		// (Efficient RMSD measures for the comparison of two molecular ensembles, 2002, Proteins: Structure, Function, and Bioinformatics)
		Trajectory t1 = trajectory.getSubTrajectory(0, 10);
		Trajectory t2 = trajectory.getSubTrajectory(10, 19);
		double eRMSD = Dynamics.getEnsembleAveragedRMSD(t1, t2);
		
		// Calculating the (residue) contact probability map for a trajectory as defined by Wei et al.
		// (Residual Structure in Islet Amyloid Polypeptide Mediates Its Interactions with Soluble Insulin, 2009, Biochemistry)
		Matrix contactProbabilityMap = Dynamics.getContactProbabilityMap(trajectory, 0.4);
		
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	
}
