/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package analysis;

import java.io.IOException;
import java.util.ArrayList;

import jama.Matrix;
import jgromacs.analysis.Dynamics;
import jgromacs.data.PointList;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;
import jgromacs.io.IOMath;
/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to perform Principal Component Analysis (PCA)
 * 
 */
public class PCA {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory; 
		
		try {
		// Reading structure and trajectory from files
		structure = IOData.readStructureFromGRO("dataset//example_calpha.gro");
	    trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_calpha.dxtc");
	
		// Calculating the NxN atomic correlation matrix of a trajectory
		// The reference frame for superposition is the first frame:
		Matrix atomicCorrelationMatrix1 = Dynamics.getAtomicCorrelationMatrix(trajectory);
		// The reference frame is also specified:
		PointList reference = trajectory.getFrameAsPointList(12); 
		Matrix atomicCorrelationMatrix2 = Dynamics.getAtomicCorrelationMatrix(trajectory, reference);
		
		// Calculating the NxN atomic covariance matrix of a trajectory
		// The reference frame for superposition is the first frame:
		Matrix atomicCovarianceMatrix1 = Dynamics.getAtomicCovarianceMatrix(trajectory);
		// The reference frame is also specified:
		Matrix atomicCovarianceMatrix2 = Dynamics.getAtomicCovarianceMatrix(trajectory, reference);
		
		// Calculating the 3Nx3N coordinate correlation matrix of a trajectory
		// The reference frame for superposition is the first frame:
		Matrix coordinateCorrelationMatrix1 = Dynamics.getCoordinateCorrelationMatrix(trajectory);
		// The reference frame is also specified:
		Matrix coordinateCorrelationMatrix2 = Dynamics.getCoordinateCorrelationMatrix(trajectory, reference);
		
		// Calculating the 3Nx3N coordinate covariance matrix of a trajectory
		// The reference frame for superposition is the first frame:
		Matrix coordinateCovarianceMatrix1 = Dynamics.getCoordinateCovarianceMatrix(trajectory);
		// The reference frame is also specified:
		Matrix coordinateCovarianceMatrix2 = Dynamics.getCoordinateCovarianceMatrix(trajectory, reference);
		
		// Performing Principal Component Analysis (PCA)
		Matrix[] PCAresult = Dynamics.getPCA(coordinateCovarianceMatrix2); // Using a pre-calculated covariance matrix
		Matrix[] PCAresult2 = Dynamics.getPCA(trajectory);  // Using a trajectory as input
		Matrix D = PCAresult[0]; // block diagonal eigenvalue matrix
		Matrix V = PCAresult[1]; // principal component matrix
				
		// Calculating the cumulative variance profile
		ArrayList<Double> cumulativeVarianceProfile1 = Dynamics.getCumulativeVariances(coordinateCovarianceMatrix1); // Using a pre-calculated covariance matrix
		ArrayList<Double> cumulativeVarianceProfile2 = Dynamics.getCumulativeVariances(trajectory); // Using a trajectory as input
		
		// Calculating the Root Mean Square Inner Product (RMSIP) between two dynamic subspaces
		// Using pre-calculated covariance matrices:
		double RMSIP1 = Dynamics.getRootMeanSquareInnerProduct(coordinateCovarianceMatrix1, coordinateCovarianceMatrix2, 10, 10); // 10 PCs used for both trajectories
		// Using trajectories as input:
		double RMSIP2 = Dynamics.getRootMeanSquareInnerProduct(trajectory, trajectory, 10, 14); // 10 PCs used for the first and 14 PCs used for the second
		
		// Calculating the covariance matrix overlap
		double overlap1 = Dynamics.getCovarianceMatrixOverlap(coordinateCovarianceMatrix1, coordinateCovarianceMatrix2);  // Using pre-calculated covariance matrices
		double overlap2 = Dynamics.getCovarianceMatrixOverlap(trajectory, trajectory); // Using trajectories as input
		
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
