/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package analysis;

import java.io.IOException;
import java.util.ArrayList;

import jama.Matrix;
import jgromacs.analysis.Distances;
import jgromacs.analysis.Similarity;
import jgromacs.data.FrameIndexSet;
import jgromacs.data.IndexSet;
import jgromacs.data.PointList;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;
/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to calculate similarity of conformations
 * 
 */
public class Similarities {

	public static void main(String[] args) {
		
		Structure structure;
		Trajectory trajectory;
		
		try {
		// Reading structure and trajectory from files
		structure = IOData.readStructureFromGRO("dataset//example_calpha.gro");
		trajectory = IOData.readDumpedTrajectory(structure,"dataset//example_calpha.dxtc");
		
		Structure structureA = trajectory.getFrameAsStructure(4); // selecting first structure
		Structure structureB = trajectory.getFrameAsStructure(8); // selecting second structure
		
		PointList pointlistA = trajectory.getFrameAsPointList(4); // selecting first point list
		PointList pointlistB = trajectory.getFrameAsPointList(8); // selecting second point list
		
		IndexSet group = structureA.getAlphaCarbonIndexSet(); // selecting index group
		
		// Calculating RMSD similarity
		// between two point lists:
		double RMSD1 = Similarity.getRMSD(pointlistA, pointlistB); 
		// between two structures:
		double RMSD2 = Similarity.getRMSD(structureA, structureB); 
		// between two groups of atoms:
		double RMSD3 = Similarity.getRMSD(structureA, group, structureB, group); 
		
		// Calculating RMSD similarity without performing superposition
		// between two point lists:
		double RMSD4 = Similarity.getRMSDnoSuperposition(pointlistA, pointlistB); 
		// between two structures:
		double RMSD5 = Similarity.getRMSDnoSuperposition(structureA, structureB);  
		// between two groups of atoms:
	    double RMSD6 = Similarity.getRMSDnoSuperposition(structureA, group, structureB, group); 
	    
	    // Calculating RMSDi profile
	    // between two point lists:
	    ArrayList<Double> RMSDi_profile1 = Similarity.getRMSDiProfile(pointlistA, pointlistB); 
	    // between two structures:
	    ArrayList<Double> RMSDi_profile2 = Similarity.getRMSDiProfile(structureA, structureB); 
	    // between two groups of atoms:
	    ArrayList<Double> RMSDi_profile3 = Similarity.getRMSDiProfile(structureA, group, structureB, group); 
	    
	    // Calculating RMSDi profile without performing superposition:
	    // between two point lists:
	    ArrayList<Double> RMSDi_profile4 = Similarity.getRMSDiProfileNoSuperposition(pointlistA, pointlistB);
	    // between two structures:
	    ArrayList<Double> RMSDi_profile5 = Similarity.getRMSDiProfileNoSuperposition(structureA, structureB); 
	    // between two groups of atoms:
	    ArrayList<Double> RMSDi_profile6 = Similarity.getRMSDiProfileNoSuperposition(structureA, group, structureB, group); 
	    
	    // Calculating the dRMSD similarity
	    // between two distance matrices:
	    Matrix A = Distances.getDistanceMatrix(pointlistA);
	    Matrix B = Distances.getDistanceMatrix(pointlistB);
	    double dRMSD1 = Similarity.getDRMSD(A, B);  
	    // between two point lists:
	    double dRMSD2 = Similarity.getDRMSD(pointlistA, pointlistB);
	    // between two structures:
	    double dRMSD3 = Similarity.getDRMSD(structureA, structureB);  
	    // between two groups of atoms:
	    double dRMSD4 = Similarity.getDRMSD(structureA, group, structureB, group); 
	    
	    
	    // Calculating dRMSDi profile
	    // between two distance matrices:
	    ArrayList<Double> dRMSDi_profile1 = Similarity.getDRMSDiProfile(A, B);
	    // between two point lists:
	    ArrayList<Double> dRMSDi_profile2 = Similarity.getDRMSDiProfile(pointlistA, pointlistB); 
	    // between two structures:
	    ArrayList<Double> dRMSDi_profile3 = Similarity.getDRMSDiProfile(structureA, structureB); 
	    // between two groups of atoms:
	    ArrayList<Double> dRMSDi_profile4 = Similarity.getDRMSDiProfile(structureA, group, structureB, group);
	    
	    // Calculating the wdRMSD (weighted dRMSD) similarity
	    Matrix W = Matrix.identity(pointlistA.getNumberOfPoints(), pointlistA.getNumberOfPoints()); // Weight matrix
	    // between two point lists:
	    double wdRMSD1 = Similarity.getWDRMSD(pointlistA, pointlistB, W);
	 	// between two structures:
	    double wdRMSD2 = Similarity.getWDRMSD(structureA, structureB, W);
	    // between two groups of atoms:
	    double wdRMSD3 = Similarity.getWDRMSD(structureA, group, structureB, group, W);
	    
	    // Calculating wdRMSDi (weighted dRMSDi) profile
		// between two point lists:
	    ArrayList<Double> wdRMSDi_profile1 = Similarity.getWDRMSDiProfile(pointlistA, pointlistB, W);
	 	// between two structures:
	    ArrayList<Double> wdRMSDi_profile2 = Similarity.getWDRMSDiProfile(structureA, structureB, W);
	    // between two groups of atoms:
	    ArrayList<Double> wdRMSDi_profile3 = Similarity.getWDRMSDiProfile(structureA, group, structureB, group, W);
	    
	    // Calculating the difference distance matrix
	    // between two point lists:
	    Matrix diffdist1 = Similarity.getDifferenceDistanceMatrix(pointlistA, pointlistB);
	    // between two structures:
	    Matrix diffdist2 = Similarity.getDifferenceDistanceMatrix(structureA, structureB);
	    // between two groups of atoms:
	    Matrix diffdist3 = Similarity.getDifferenceDistanceMatrix(structureA, group, structureB, group);
	    
	    
	    // Calculating the similarity matrix of all frames in a trajectory using the RMSD measure
	    // using all atoms:
	    Matrix similiarityMatrix_RMSD1 = Similarity.getSimilarityMatrixRMSD(trajectory);
	    // using a group of atoms:
	    Matrix similiarityMatrix_RMSD2 = Similarity.getSimilarityMatrixRMSD(trajectory, group);
	    
	    // Calculating the similarity matrix of all frames in a trajectory using the dRMSD measure
	    // using all atoms:
	    Matrix similiarityMatrix_dRMSD1 = Similarity.getSimilarityMatrixDRMSD(trajectory);
	    // using a group of atoms:
	    Matrix similiarityMatrix_dRMSD2 = Similarity.getSimilarityMatrixDRMSD(trajectory, group);
	    
	    // Calculating the similarity matrix of all frames in a trajectory using the wdRMSD measure
	    // using all atoms:
	    Matrix similiarityMatrix_wdRMSD1 = Similarity.getSimilarityMatrixWDRMSD(trajectory, W);
	    // using a group of atoms:
	    Matrix similiarityMatrix_wdRMSD2 = Similarity.getSimilarityMatrixWDRMSD(trajectory, group, W);
	    
	    
	    // Calculating the time series of RMSD in a trajectory with regards to a reference structure
	    Structure firstFrame = trajectory.getFirstFrameAsStructure();
	    // using all atoms:
	    ArrayList<Double> timeSeries_RMSD1 = Similarity.getSimilarityTimeSeriesRMSD(trajectory, firstFrame);
	    // using a group of atoms:
	    ArrayList<Double> timeSeries_RMSD2 = Similarity.getSimilarityTimeSeriesRMSD(trajectory, group, firstFrame, group);
	    
	    // Calculating the time series of dRMSD in a trajectory with regards to a reference structure
	    // using all atoms:
	    ArrayList<Double> timeSeries_dRMSD2 = Similarity.getSimilarityTimeSeriesDRMSD(trajectory, firstFrame);
	    // using a group of atoms:
	    ArrayList<Double> timeSeries_dRMSD3 = Similarity.getSimilarityTimeSeriesDRMSD(trajectory, group, firstFrame, group);
	
	    // Calculating the time series of wdRMSD in a trajectory with regards to a reference structure
	    // using all atoms:
	    ArrayList<Double> timeSeries_wdRMSD1 = Similarity.getSimilarityTimeSeriesWDRMSD(trajectory, firstFrame, W);
	    // using a group of atoms:
	    ArrayList<Double> timeSeries_wdRMSD3 = Similarity.getSimilarityTimeSeriesWDRMSD(trajectory, group, firstFrame, group, W);
	    
	    
	    // Calculating the medoid frame in a trajectory
	    Similarity.getMedoidRMSD(trajectory); // using the RMSD similarity measure
	    Similarity.getMedoidRMSD(trajectory); // using the dRMSD similarity measure
	    
	    
	    // Extracting the list of frames that are more similar to a reference frame than a cutoff value
	    FrameIndexSet similar_RMSD = Similarity.findSimilarFramesRMSD(trajectory, trajectory.getFrameAsPointList(10), 0.8); // RMSD similarity
	    FrameIndexSet similar_dRMSD = Similarity.findSimilarFramesRMSD(trajectory, trajectory.getFrameAsPointList(10), 0.6); // dRMSD similarity
	    
		} catch (IOException e) {
			e.printStackTrace();
		}
	    
	}
	
}
