/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package data;

import java.io.IOException;
import java.util.ArrayList;

import jgromacs.data.Alignment;
import jgromacs.data.IndexSet;
import jgromacs.data.IndexSetList;
import jgromacs.data.Sequence;
import jgromacs.data.SequencePosition;
import jgromacs.data.Structure;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;
/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to use sequences, sequence positions and alignments
 * 
 */
public class Sequences {
	
	
	public static void main(String[] args) {
		
		Sequence sequence;
		Structure structure;
		
		try {
		// Reading a sequence from FASTA file	
		sequence = IOData.readSequenceFromFASTA("dataset//example_sequence.fasta");
		
		// Extracting sequence from a structure
		structure = IOData.readStructureFromGRO("dataset//example_protein.gro");
		Sequence sequence2 = structure.getSequence();
		
		// Name
		System.out.println(sequence.getName());
		sequence.setName("seqX");
		
		// String representations
		System.out.println();
		System.out.println(sequence);
		System.out.println();
		System.out.println(sequence.toString1Letter());
		System.out.println();
		System.out.println(sequence.toString1Letter(21)); // lines split
		System.out.println();
		System.out.println(sequence.toString3Letter());
		System.out.println();
		System.out.println(sequence.toString3Letter(21)); // lines split
		
		// Summary information
		System.out.println();
		System.out.println(sequence.toStringInfo());
		
		// Sequence lenght
		System.out.println();
		System.out.println(sequence.getSequenceLength()); // without gaps
		System.out.println(sequence.getSequenceLengthWithGaps()); // with gaps
		
		// Reverse and concatenation
		Sequence sequence3 = sequence.getReverse();
		Sequence sequence4 = sequence.concat(sequence3);
		
		// Extracting sequence positions
		SequencePosition position = sequence.getPosition(7);
		SequencePosition position2 = sequence.getPositionByIndex(10);
		
		// Dealing with sequence positions
		System.out.println();
		System.out.println(position); // String representation
		System.out.println(position.getIndex()); // Index;
		System.out.println(position.getAnnotation()); // Annotation;
		System.out.println(position.getResidueType()); // Residue Type;
		position.setIndex(20);
		position.setAnnotation("H");
		position.setResidueType(new ResidueType("GLY"));
		
		// Getting properties of sequence positions from the Sequence object
		ResidueType type = sequence.getResidueTypeOfPosition(11);
		System.out.println(sequence.getAnnotationOfPositionOfIndex(11));
		System.out.println(sequence.getIndexOfPosition(14));
		
		// Extracting sequence positions as an ArrayList object
		ArrayList<SequencePosition> positions = sequence.getAsArrayList();
		
		// Adding, setting and removing sequence positions
		SequencePosition pos = new SequencePosition();
		sequence.addPosition(pos);
		sequence.setPosition(7, pos);
		sequence.setPosition(12, 67, new ResidueType("GLY"), "H");
		sequence.removePosition(9);
		sequence.removePositionByIndex(19);
		
		// Adding new sequence positions from a String object
		String str = "GGGGGHHHHHGHHH";
		sequence.addPositionsFromString(str);
		
		// Dealing with gaps
		boolean isGap = sequence.isPositionAGap(15); // is this position a gap?
		sequence.addGap(); // new gap added
		sequence.insertGap(7); // new gap inserted
		sequence.removeGaps(); // all gaps removed
	
		// Re-indexing sequence positions
		sequence.reIndexPositions(); // Starting from 1
		sequence.reIndexPositions(66); // Starting from 66
		
		// Extracting subsequences
		Sequence subs = sequence.getSubSequence(6, 14);
		Sequence subs2 = sequence.getSubSequenceFrom(3);
		Sequence subs3 = sequence.getSubSequenceTo(39);
		
		// Reading an alignment from FASTA file
		Alignment alignment = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
		
		// String representation
		System.out.println();
		System.out.println(alignment);
		// Summary information
		System.out.println();
		System.out.println(alignment.toStringInfo());
		
		// Number of sequences
        System.out.println(alignment.getNumberOfSequences());
          
        // Extracting sequences
        Sequence sequence5 = alignment.getSequence(3);
        
        // Sequence lengths
        System.out.println(alignment.getMaxSequenceLength());
        System.out.println(alignment.getMinSequenceLength());
        
        // Adding, setting and removing sequences
        alignment.addSequence(sequence2);
        alignment.setSequence(3, sequence2);
        alignment.removeSequence(1);
      
        // Extracting sequences as an ArrayList object
        ArrayList<Sequence> sequences = alignment.getSequencesAsArrayList();
		
        // Columns in the alignment
        alignment.removeColumn(8); // Removing a column
        boolean isMatchColumn = alignment.isMatchColumn(10); // no gaps in the column?
      
        // Extracting consensus sequence
        alignment = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta"); // reloading alignment from file
        Sequence consensus = alignment.getConsensusSequence();
        System.out.println();
        System.out.println(consensus); // Writing out consensus alignment
        ResidueType mostfrequent = alignment.getMostFrequentResidueType(20);
        
        // Extracting collapsed alignment
        Alignment collapsed = alignment.getCollapsedAlignment(); // keep only match columns
        System.out.println();
        System.out.println(collapsed); // Writing out collapsed alignment
        
        // Extracting match position indices
        IndexSet matchPositionIndices = alignment.getMatchPositionIndices(2); // Match position indices in sequence 2
        IndexSetList matchPositionIndexGroups = alignment.getMatchPositionIndexSets();  // Match position index groups for each sequences
	
        // Cloning sequences and alignments
        Sequence cseq = (Sequence)sequence.clone();
        Alignment calign = (Alignment)alignment.clone();
		
		} catch (IOException e) {
			e.printStackTrace();
		}    
	}
	
}
