/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package data;

import java.io.IOException;

import jgromacs.data.Atom;
import jgromacs.data.IndexSet;
import jgromacs.data.IndexSetList;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Sequence;
import jgromacs.data.Structure;
import jgromacs.db.AtomType;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;

/**
 * 
 *  Example code for the JGromacs v1.0. package
 *  Topic: How to use structures, residues and atoms
 * 
 */
public class Structures {

	
	public static void main(String[] args) {
		
		Structure structure;
		
		try {
		// Reading structure from from file
		structure = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
		// Printing out String representations
		System.out.println(structure);
		System.out.println(structure.toStringAsGRO());
		System.out.println(structure.toStringAsPDB());
		// Printing out summary information
		System.out.println();
		System.out.println(structure.toStringInfo());
		
		// Number of residues
		System.out.println();
		System.out.println(structure.getNumberOfResidues());
		// Number of atoms
		System.out.println(structure.getNumberOfAtoms());
		
		// Extracting residues from a structure
		Residue residue = structure.getResidue(2);
		Residue residue2 = structure.getResidueByIndex(1312);
		Residue residue3 = structure.getResidueByIndex(1402, "A"); // Residue 12 in chain A
		// Extracting atoms from a structure
		Atom atom = structure.getAtom(42);
		Atom atom2 = structure.getAtomByIndex(50);
		Atom atom3 = structure.getAtomInResidueOfIndex(1312, 4); // Atom #2 in residue 20
		
		// Basics of residues
		System.out.println();
		System.out.println(residue); // String representation
		System.out.println(residue.toStringInfo()); // Summary information
		System.out.println(residue.getName()); // Name
		System.out.println(residue.getIndex()); // Index
		System.out.println(residue.getChainID()); // Chain ID
		System.out.println(residue.get1LetterCode()); // 1-letter code
		System.out.println(residue.get3LetterCode()); // 3-letter code
		System.out.println(residue.getCombinedCode()); // Combined code
		Point3D point = residue.getAlphaCarbonCoordinates(); // Alpha-Carbon coordinates
		ResidueType type = residue.getResidueType(); // Residue type
		boolean isAminoAcid = residue.isAminoAcid(); // Is it an Amino Acid?
		boolean isWater = residue.isWater(); // Is it a water molecule?
		
		// Basics of atoms
		System.out.println();
		System.out.println(atom); // String representation
		System.out.println(atom.getName()); // Name
		System.out.println(atom.getIndex()); // Index
		Point3D coord = atom.getCoordinates(); // Coordinates
		System.out.println(atom.getZCoordinate()); // Z-coordinate
		AtomType type2 = atom.getAtomType();  // Atom type
		boolean isAlphaCarbon = atom.isAlphaCarbon(); // is it an alpha-carbon atom?
		boolean isCarbonylOxygen = atom.isCarbonylOxygen(); // is it a carbonyl oxygen atom?
		
		// Modifying residues and atoms
		residue.setIndex(111);
		residue.setChainID("B");
		residue.setResidueType(new ResidueType("HIS"));
		atom.setName("CA");
		atom.setIndex(333);
		atom.setXCoordinate(10.3);
		atom.setAtomType(new AtomType("Zn"));
						
		// Extracting atoms from a residue
		Atom atom4 = residue.getAtom(3);
		Atom atom5 = residue.getAtomByIndex(14);
		Atom atom6 = residue.getAlphaCarbon();
		Atom atom7 = residue.getBetaCarbon();
		Atom atom8 = residue.getNTerminalNitrogen();
				
		
		// Calculating the distance between atoms
		double distance = atom4.distance(atom5);
		
		// Calculating the distance between residues
		double distance2 = residue.distanceAlphaCarbons(residue2); // distance between alpha-carbon atoms
		double distance3 = residue.distanceClosest(residue2); // distance of the closest atoms
		double distance4 = residue.distanceClosestHeavy(residue2); // distance of the closest heavy atoms
		
		// Extracting index groups from a structure
		IndexSet sideChainIndexGroup = structure.getSideChainIndexSet(); // side chain atoms
		IndexSet heavyProteinAtomsIndexGroup = structure.getHeavyProteinIndexSet(); // heavy protein atoms
		IndexSet alphaCarbonIndexGroup = structure.getAlphaCarbonIndexSet();  // alpha-carbon atoms 
		IndexSet mainChainPlusHIndexGroup = structure.getMainChainPlusHIndexSet(); // main chain plus H atoms
		// Extracting the default index group list from a structure
		IndexSetList defaultgroups = structure.getDefaultIndexSetList();
		
		// Extracting amino acid sequence from a structure
		Sequence sequence = structure.getSequence();
		
		// Adding new residues and atoms to a structure
		structure.addResidue(new Residue());
		structure.addAtomToResidue(5, new Atom());
		structure.addAtomToResidueOfIndex(18, new Atom());
		// Removing residues and atoms from a structure
		structure.removeResidue(22);
		structure.removeResidue(residue2);
		structure.removeAtom(atom8);
		structure.removeAtom(1);
		
		// Extracting and setting atomic coordinates
		PointList points = structure.getAllAtomCoordinates();
		structure.setAtomCoordinates(7, new Point3D(1,2,3));
		structure.setAllAtomCoordinates(points);
		
		// Extracting a substructure
		IndexSet indices = structure.getBackboneIndexSet();
		Structure substructure = structure.getSubStructure(indices);
		
		// cloning structures, residues and atoms
		Structure sclone = (Structure)structure.clone();
		Residue rclone = (Residue)residue.clone();
		Atom aclone = (Atom)atom.clone();
		
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
